// DOCTORS.H  - Class definitions for Doctor Schedule application


#ifndef DOCTORS_H
#define DOCTORS_H

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include "fstring.h"    // custom string class

const unsigned NumDoctors = 5;
extern int getYN( const char * );

class Doctor; // forward declaration required
class Patient;

class TimeSlot {
public:
  TimeSlot( const unsigned n = 0 );

  unsigned AsInteger() const;

  friend istream & operator >>(istream & inp, TimeSlot & T);

  friend ostream & operator <<(ostream & os, const TimeSlot & T);

private:
  static unsigned StartHour;
  static unsigned ApptLen;
  unsigned intValue;
};

class Appointment {
public:
  Appointment();

  Appointment ( const TimeSlot & aTime,
       unsigned docNum, const Patient & P);

  const FString & GetPatientName() const;

  const TimeSlot & GetTime() const;

  int IsScheduled() const;

  void SetTime( const unsigned n );

  friend ostream & operator <<( ostream & os,
         const Appointment & A );

private:
  enum { NoDoctor = 9999 };
  unsigned doctorNum;
  TimeSlot timeSlot;
  FString patientName;
};

class Patient {
public:
  Patient();

  void InputName();

  unsigned ChooseDoctor() const;

  TimeSlot ChooseTimeSlot(const Doctor & D) const;

  const Appointment & GetAppointment() const;

  const FString & GetFirstName() const;

  const FString & GetLastName() const;

  int IsScheduled() const;

  void SetAppointment( const Appointment & A );

  friend ostream & operator <<( ostream & os,
     const Patient & P );

private:
  FString lastName;
  FString firstName;
  Appointment nextVisit;
};

class DailySchedule {
public:
  DailySchedule();

  int IsTimeSlotFree( const TimeSlot & T ) const;

  void SetAppointment( const Appointment & A );

  void ShowAppointments( ostream & os ) const;

  friend ostream & operator <<( ostream & os,
         const DailySchedule & DS );

private:
  enum { MaxTimeSlots = 40 };
  Appointment appointments[MaxTimeSlots];
};

class Doctor {
public:
  Doctor();

  int AddToSchedule( const Appointment & A );
  // Try to schedule an appointment. Return 1 (true) if
  // successful, 0 (false) if not.

  const DailySchedule & GetSchedule() const;

  void SetId( const unsigned );

  void SetLastName( const FString & L );

  const FString & GetLastName() const;

  void ShowAppointments( ostream & ) const;

  static const FString & GetDoctorName( unsigned index );

  static void SetDoctorName(unsigned, const FString & nam);

private:
  unsigned id;
  FString lastName;
  DailySchedule schedule;
  static FString doctorName[NumDoctors];
};


class Scheduler {
public:
  Scheduler( Doctor * docs );

  void PrintAllAppointments( const char * fileName );

  int ScheduleOneAppointment();

  void ScheduleAllAppointments();

private:
  Doctor * doctors;  // points to first element in array of doctors
};

//............. Inline functions ..................

inline TimeSlot::TimeSlot( const unsigned n )
{
  intValue = n;
}

inline unsigned TimeSlot::AsInteger() const
{
  return intValue;
}

inline const DailySchedule & Doctor::GetSchedule() const
{
  return schedule;
}

inline void Doctor::SetId( const unsigned n )
{
 id = n;
}

inline void Doctor::SetLastName( const FString & aName )
{
  lastName = aName;
}

inline const FString & Doctor::GetLastName() const
{
  return lastName;
}

inline const FString & Doctor::GetDoctorName( unsigned i )
{
  return doctorName[i];
}

inline void Doctor::SetDoctorName( unsigned i, const FString & aName )
{
  doctorName[i] = aName;
}

inline const FString & Appointment::GetPatientName() const
{
  return patientName;
}

inline int Appointment::IsScheduled() const
{
  return doctorNum != NoDoctor;
}

inline void Appointment::SetTime( const unsigned n )
{
  timeSlot = n;
}

inline const TimeSlot & Appointment::GetTime() const
{
  return timeSlot;
}

inline const FString & Patient::GetFirstName() const
{
  return firstName;
}

inline const FString & Patient::GetLastName() const
{
  return lastName;
}

inline Appointment::Appointment()
{
  doctorNum = NoDoctor;
}

inline const Appointment & Patient::GetAppointment() const
{
  return nextVisit;
}

inline Patient::Patient() {  }

inline void Patient::SetAppointment( const Appointment & app )
{
  nextVisit = app;
}

inline int Patient::IsScheduled() const
{
  return nextVisit.IsScheduled();
}


#endif
